/*
 * Copyright (c) eHealth
 */
package be.ehealth.businessconnector.test.mediprima.session;


import be.ehealth.businessconnector.mediprima.session.MediprimaSessionServiceFactory;
import be.ehealth.businessconnector.test.mediprima.builder.ConsultCarmedInterventionRequestBuilder;
import be.ehealth.technicalconnector.exception.SoaErrorException;
import be.ehealth.technicalconnector.utils.DateUtils;
import be.ehealth.technicalconnector.utils.MarshallerHelper;
import be.ehealth.technicalconnector.utils.TemplateEngineUtils;
import be.fgov.ehealth.mediprima.protocol.v1.ConsultCarmedInterventionRequest;
import be.fgov.ehealth.mediprima.protocol.v1.ConsultCarmedInterventionResponse;
import be.fgov.ehealth.technicalconnector.tests.junit.rule.RestoreSystemProperties;
import be.fgov.ehealth.technicalconnector.tests.junit.rule.SessionRule;
import be.fgov.ehealth.technicalconnector.tests.utils.XmlAsserter;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.junit.Assert;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.experimental.runners.Enclosed;
import org.junit.rules.RuleChain;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.fail;

/**
 * Integration tests for consultation mediprima persphysician. The scenarios numbering match those of the MediPrima specification.
 *
 * @author EHP
 */
@RunWith(Enclosed.class)
public final class MediprimaConsultationPersphysicianIntegrationTest {

    private static final Logger LOG = LoggerFactory.getLogger(MediprimaConsultationPersphysicianIntegrationTest.class);


    @RunWith(Parameterized.class)
    public static class WithSoaErrorIntegrationTest {
        @ClassRule
        public static SessionRule rule = SessionRule.withActiveSession().forEnvironment("acc").forProfession("persphysician").forUser("hannes").build();
        private String scenarioId;
        private String scenarioName;
        private ConsultCarmedInterventionRequest request;


        public WithSoaErrorIntegrationTest(String scenarioId, String scenarioName, ConsultCarmedInterventionRequest request) {
            super();
            this.scenarioId = scenarioId;
            this.scenarioName = scenarioName;
            this.request = request;
        }

        @Parameterized.Parameters(name = "{0}: {1}")
        public static Collection<Object[]> data() throws Exception {
            Collection<Object[]> testData = new ArrayList<Object[]>();
            testData.add(new Object[]{"scenario01", "Invalid ssin", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("85000132773").withPeriod().build()});
            testData.add(new Object[]{"scenario02", "Consult with physician and period", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("86081523513").withPeriod().build()});
            testData.add(new Object[]{"scenario03", "Consultation date > today", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("86081523513").withReferenceDate(new DateTime().plusDays(10)).build()});
            testData.add(new Object[]{"scenario04", "Consultation date > today", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("86081523513").withReferenceDate(new DateTime().minusMonths(3)).build()});

            return testData;
        }

        @Test
        public void testScenario() throws Exception {
            try {
                MediprimaSessionServiceFactory.getConsultationMediprimaSession().consultCarmedIntervention(request);
                fail("should have thrown SoaErrorException");
            } catch (SoaErrorException e) {
                ConsultCarmedInterventionResponse response = (ConsultCarmedInterventionResponse) e.getResponseTypeV2();
                String expected = expectedResponse(request.getSelectionCriteria().getBySsin().getReferenceDate(), request.getId(), scenarioId);
                MarshallerHelper<ConsultCarmedInterventionResponse, ConsultCarmedInterventionResponse> helper = new MarshallerHelper<ConsultCarmedInterventionResponse, ConsultCarmedInterventionResponse>(ConsultCarmedInterventionResponse.class, ConsultCarmedInterventionResponse.class);
                XmlAsserter.assertSimilar(expected, helper.toString(response));
            }
        }
    }

    @RunWith(Parameterized.class)
    public static class WithoutSoaErrorIntegrationTest {

        @ClassRule
        public static RuleChain chain = RuleChain.outerRule(new RestoreSystemProperties()).around(SessionRule.withActiveSession().forEnvironment("acc").forProfession("persphysician").forUser("hannes").build());


        private String scenarioId;
        private String scenarioName;
        private ConsultCarmedInterventionRequest request;


        public WithoutSoaErrorIntegrationTest(String scenarioId, String scenarioName, ConsultCarmedInterventionRequest request) {
            super();
            this.scenarioId = scenarioId;
            this.scenarioName = scenarioName;
            this.request = request;
        }


        @Parameterized.Parameters(name = "{0}: {1}")
        public static Collection<Object[]> data() throws Exception {
            Collection<Object[]> testData = new ArrayList<Object[]>();
            testData.add(new Object[]{"scenario05", "Unknown ssin", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("86081523513").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario06", "Known ssin but dates nok", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("35061540420").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario07a", "Decision without required rights - missing coverage", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("35061540420").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario07b", "Decision without required rights - missing coverage", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("76091051964").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario08", "Decision without required rights - coverage exists but nok", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("61011060097").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario09", "Decision without required rights - coverage ok but care provider nok", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("74052543021").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario10", "Decision pending", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("11030441141").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario11", "Decision is active - full support", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("92010442932").withReferenceDate(new DateTime()).build()});
            testData.add(new Object[]{"scenario12", "Decision is active - partial support", ConsultCarmedInterventionRequestBuilder.aRequest().withSsin("75052657066").withReferenceDate(new DateTime()).build()});

            return testData;
        }

        @Test
        public void testScenario() throws Exception {
            ConsultCarmedInterventionResponse response = null;
            try {
                response = MediprimaSessionServiceFactory.getConsultationMediprimaSession().consultCarmedIntervention(request);
            } catch (SoaErrorException e) {
                fail(e.getErrorCode());
            }

            String expected = expectedResponse(request.getSelectionCriteria().getBySsin().getReferenceDate(), request.getId(), scenarioId);
            MarshallerHelper<ConsultCarmedInterventionResponse, ConsultCarmedInterventionResponse> helper = new MarshallerHelper<ConsultCarmedInterventionResponse, ConsultCarmedInterventionResponse>(ConsultCarmedInterventionResponse.class, ConsultCarmedInterventionResponse.class);
            XmlAsserter.assertSimilar(expected, helper.toString(response));
        }
    }

    private static String expectedResponse(DateTime consultationDate, String requestId, String scenarioId) {
        Map<String, Object> velocityContext = new HashMap<String, Object>();
        velocityContext.put("requestId", requestId);
        velocityContext.put("consultationDate", DateUtils.printDate(consultationDate));
        String commentDate = DateTimeFormat.forPattern("dd/MM/yyyy").print(new DateTime());
        velocityContext.put("commentStartDate", commentDate);
        velocityContext.put("commentEndDate", commentDate);
        velocityContext.put("startDate", DateUtils.printDate(new DateTime()));
        velocityContext.put("endDate", DateUtils.printDate(new DateTime()));
        return TemplateEngineUtils.generate(velocityContext, "/consultation/expected/" + scenarioId + "Response.xml");
    }
}
